# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: ibc/core/channel/v1/channel.proto, ibc/core/channel/v1/genesis.proto, ibc/core/channel/v1/query.proto, ibc/core/channel/v1/tx.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from .....cosmos.base.query import v1beta1 as ____cosmos_base_query_v1_beta1__
from ...client import v1 as __client_v1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class State(betterproto.Enum):
    """
    State defines if a channel is in one of the following states:
    CLOSED, INIT, TRYOPEN, OPEN or UNINITIALIZED.
    """

    STATE_UNINITIALIZED_UNSPECIFIED = 0
    """Default State"""

    STATE_INIT = 1
    """A channel has just started the opening handshake."""

    STATE_TRYOPEN = 2
    """
    A channel has acknowledged the handshake step on the counterparty chain.
    """

    STATE_OPEN = 3
    """
    A channel has completed the handshake. Open channels are
    ready to send and receive packets.
    """

    STATE_CLOSED = 4
    """
    A channel has been closed and can no longer be used to send or receive
    packets.
    """


class Order(betterproto.Enum):
    """Order defines if a channel is ORDERED or UNORDERED"""

    ORDER_NONE_UNSPECIFIED = 0
    """zero-value for channel ordering"""

    ORDER_UNORDERED = 1
    """
    packets can be delivered in any order, which may differ from the order in
    which they were sent.
    """

    ORDER_ORDERED = 2
    """packets are delivered exactly in the order which they were sent"""


@dataclass(eq=False, repr=False)
class Channel(betterproto.Message):
    """
    Channel defines pipeline for exactly-once packet delivery between specific
    modules on separate blockchains, which has at least one end capable of
    sending packets and one end capable of receiving packets.
    """

    state: 'State' = betterproto.enum_field(1)
    """current state of the channel end"""

    ordering: 'Order' = betterproto.enum_field(2)
    """whether the channel is ordered or unordered"""

    counterparty: 'Counterparty' = betterproto.message_field(3)
    """counterparty channel end"""

    connection_hops: List[str] = betterproto.string_field(4)
    """
    list of connection identifiers, in order, along which packets sent on
    this channel will travel
    """

    version: str = betterproto.string_field(5)
    """opaque channel version, which is agreed upon during the handshake"""


@dataclass(eq=False, repr=False)
class IdentifiedChannel(betterproto.Message):
    """
    IdentifiedChannel defines a channel with additional port and channel
    identifier fields.
    """

    state: 'State' = betterproto.enum_field(1)
    """current state of the channel end"""

    ordering: 'Order' = betterproto.enum_field(2)
    """whether the channel is ordered or unordered"""

    counterparty: 'Counterparty' = betterproto.message_field(3)
    """counterparty channel end"""

    connection_hops: List[str] = betterproto.string_field(4)
    """
    list of connection identifiers, in order, along which packets sent on
    this channel will travel
    """

    version: str = betterproto.string_field(5)
    """opaque channel version, which is agreed upon during the handshake"""

    port_id: str = betterproto.string_field(6)
    """port identifier"""

    channel_id: str = betterproto.string_field(7)
    """channel identifier"""


@dataclass(eq=False, repr=False)
class Counterparty(betterproto.Message):
    """Counterparty defines a channel end counterparty"""

    port_id: str = betterproto.string_field(1)
    """
    port on the counterparty chain which owns the other end of the channel.
    """

    channel_id: str = betterproto.string_field(2)
    """channel end on the counterparty chain"""


@dataclass(eq=False, repr=False)
class Packet(betterproto.Message):
    """
    Packet defines a type that carries data across different chains through IBC
    """

    sequence: int = betterproto.uint64_field(1)
    """
    number corresponds to the order of sends and receives, where a Packet
    with an earlier sequence number must be sent and received before a Packet
    with a later sequence number.
    """

    source_port: str = betterproto.string_field(2)
    """identifies the port on the sending chain."""

    source_channel: str = betterproto.string_field(3)
    """identifies the channel end on the sending chain."""

    destination_port: str = betterproto.string_field(4)
    """identifies the port on the receiving chain."""

    destination_channel: str = betterproto.string_field(5)
    """identifies the channel end on the receiving chain."""

    data: bytes = betterproto.bytes_field(6)
    """actual opaque bytes transferred directly to the application module"""

    timeout_height: '__client_v1__.Height' = betterproto.message_field(7)
    """block height after which the packet times out"""

    timeout_timestamp: int = betterproto.uint64_field(8)
    """block timestamp (in nanoseconds) after which the packet times out"""


@dataclass(eq=False, repr=False)
class PacketState(betterproto.Message):
    """
    PacketState defines the generic type necessary to retrieve and store
    packet commitments, acknowledgements, and receipts.
    Caller is responsible for knowing the context necessary to interpret this
    state as a commitment, acknowledgement, or a receipt.
    """

    port_id: str = betterproto.string_field(1)
    """channel port identifier."""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier."""

    sequence: int = betterproto.uint64_field(3)
    """packet sequence."""

    data: bytes = betterproto.bytes_field(4)
    """embedded data that represents packet state."""


@dataclass(eq=False, repr=False)
class Acknowledgement(betterproto.Message):
    """
    Acknowledgement is the recommended acknowledgement format to be used by
    app-specific protocols.
    NOTE: The field numbers 21 and 22 were explicitly chosen to avoid accidental
    conflicts with other protobuf message formats used for acknowledgements.
    The first byte of any message with this format will be the non-ASCII values
    `0xaa` (result) or `0xb2` (error). Implemented as defined by ICS:
    https://github.com/cosmos/ibc/tree/main/spec/core/ics-004-channel-and-packet-semantics
    """

    result: bytes = betterproto.bytes_field(21, group='response')
    error: str = betterproto.string_field(22, group='response')


@dataclass(eq=False, repr=False)
class QueryChannelRequest(betterproto.Message):
    """
    QueryChannelRequest is the request type for the Query/Channel RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""


@dataclass(eq=False, repr=False)
class QueryChannelResponse(betterproto.Message):
    """
    QueryChannelResponse is the response type for the Query/Channel RPC method.
    Besides the Channel end, it includes a proof and the height from which the
    proof was retrieved.
    """

    channel: 'Channel' = betterproto.message_field(1)
    """channel associated with the request identifiers"""

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryChannelsRequest(betterproto.Message):
    """
    QueryChannelsRequest is the request type for the Query/Channels RPC method
    """

    pagination: '____cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(1)
    )
    """pagination request"""


@dataclass(eq=False, repr=False)
class QueryChannelsResponse(betterproto.Message):
    """
    QueryChannelsResponse is the response type for the Query/Channels RPC method.
    """

    channels: List['IdentifiedChannel'] = betterproto.message_field(1)
    """list of stored channels of the chain."""

    pagination: '____cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination response"""

    height: '__client_v1__.Height' = betterproto.message_field(3)
    """query block height"""


@dataclass(eq=False, repr=False)
class QueryConnectionChannelsRequest(betterproto.Message):
    """
    QueryConnectionChannelsRequest is the request type for the
    Query/QueryConnectionChannels RPC method
    """

    connection: str = betterproto.string_field(1)
    """connection unique identifier"""

    pagination: '____cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(2)
    )
    """pagination request"""


@dataclass(eq=False, repr=False)
class QueryConnectionChannelsResponse(betterproto.Message):
    """
    QueryConnectionChannelsResponse is the Response type for the
    Query/QueryConnectionChannels RPC method
    """

    channels: List['IdentifiedChannel'] = betterproto.message_field(1)
    """list of channels associated with a connection."""

    pagination: '____cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination response"""

    height: '__client_v1__.Height' = betterproto.message_field(3)
    """query block height"""


@dataclass(eq=False, repr=False)
class QueryChannelClientStateRequest(betterproto.Message):
    """
    QueryChannelClientStateRequest is the request type for the Query/ClientState
    RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""


@dataclass(eq=False, repr=False)
class QueryChannelClientStateResponse(betterproto.Message):
    """
    QueryChannelClientStateResponse is the Response type for the
    Query/QueryChannelClientState RPC method
    """

    identified_client_state: '__client_v1__.IdentifiedClientState' = (
        betterproto.message_field(1)
    )
    """client state associated with the channel"""

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryChannelConsensusStateRequest(betterproto.Message):
    """
    QueryChannelConsensusStateRequest is the request type for the
    Query/ConsensusState RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""

    revision_number: int = betterproto.uint64_field(3)
    """revision number of the consensus state"""

    revision_height: int = betterproto.uint64_field(4)
    """revision height of the consensus state"""


@dataclass(eq=False, repr=False)
class QueryChannelConsensusStateResponse(betterproto.Message):
    """
    QueryChannelClientStateResponse is the Response type for the
    Query/QueryChannelClientState RPC method
    """

    consensus_state: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(
        1
    )
    """consensus state associated with the channel"""

    client_id: str = betterproto.string_field(2)
    """client ID associated with the consensus state"""

    proof: bytes = betterproto.bytes_field(3)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(4)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryPacketCommitmentRequest(betterproto.Message):
    """
    QueryPacketCommitmentRequest is the request type for the
    Query/PacketCommitment RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""

    sequence: int = betterproto.uint64_field(3)
    """packet sequence"""


@dataclass(eq=False, repr=False)
class QueryPacketCommitmentResponse(betterproto.Message):
    """
    QueryPacketCommitmentResponse defines the client query response for a packet
    which also includes a proof and the height from which the proof was
    retrieved
    """

    commitment: bytes = betterproto.bytes_field(1)
    """packet associated with the request fields"""

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryPacketCommitmentsRequest(betterproto.Message):
    """
    QueryPacketCommitmentsRequest is the request type for the
    Query/QueryPacketCommitments RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""

    pagination: '____cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(3)
    )
    """pagination request"""


@dataclass(eq=False, repr=False)
class QueryPacketCommitmentsResponse(betterproto.Message):
    """
    QueryPacketCommitmentsResponse is the request type for the
    Query/QueryPacketCommitments RPC method
    """

    commitments: List['PacketState'] = betterproto.message_field(1)
    pagination: '____cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination response"""

    height: '__client_v1__.Height' = betterproto.message_field(3)
    """query block height"""


@dataclass(eq=False, repr=False)
class QueryPacketReceiptRequest(betterproto.Message):
    """
    QueryPacketReceiptRequest is the request type for the
    Query/PacketReceipt RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""

    sequence: int = betterproto.uint64_field(3)
    """packet sequence"""


@dataclass(eq=False, repr=False)
class QueryPacketReceiptResponse(betterproto.Message):
    """
    QueryPacketReceiptResponse defines the client query response for a packet
    receipt which also includes a proof, and the height from which the proof was
    retrieved
    """

    received: bool = betterproto.bool_field(2)
    """success flag for if receipt exists"""

    proof: bytes = betterproto.bytes_field(3)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(4)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryPacketAcknowledgementRequest(betterproto.Message):
    """
    QueryPacketAcknowledgementRequest is the request type for the
    Query/PacketAcknowledgement RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""

    sequence: int = betterproto.uint64_field(3)
    """packet sequence"""


@dataclass(eq=False, repr=False)
class QueryPacketAcknowledgementResponse(betterproto.Message):
    """
    QueryPacketAcknowledgementResponse defines the client query response for a
    packet which also includes a proof and the height from which the
    proof was retrieved
    """

    acknowledgement: bytes = betterproto.bytes_field(1)
    """packet associated with the request fields"""

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class QueryPacketAcknowledgementsRequest(betterproto.Message):
    """
    QueryPacketAcknowledgementsRequest is the request type for the
    Query/QueryPacketCommitments RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""

    pagination: '____cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(3)
    )
    """pagination request"""

    packet_commitment_sequences: List[int] = betterproto.uint64_field(4)
    """list of packet sequences"""


@dataclass(eq=False, repr=False)
class QueryPacketAcknowledgementsResponse(betterproto.Message):
    """
    QueryPacketAcknowledgemetsResponse is the request type for the
    Query/QueryPacketAcknowledgements RPC method
    """

    acknowledgements: List['PacketState'] = betterproto.message_field(1)
    pagination: '____cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination response"""

    height: '__client_v1__.Height' = betterproto.message_field(3)
    """query block height"""


@dataclass(eq=False, repr=False)
class QueryUnreceivedPacketsRequest(betterproto.Message):
    """
    QueryUnreceivedPacketsRequest is the request type for the
    Query/UnreceivedPackets RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""

    packet_commitment_sequences: List[int] = betterproto.uint64_field(3)
    """list of packet sequences"""


@dataclass(eq=False, repr=False)
class QueryUnreceivedPacketsResponse(betterproto.Message):
    """
    QueryUnreceivedPacketsResponse is the response type for the
    Query/UnreceivedPacketCommitments RPC method
    """

    sequences: List[int] = betterproto.uint64_field(1)
    """list of unreceived packet sequences"""

    height: '__client_v1__.Height' = betterproto.message_field(2)
    """query block height"""


@dataclass(eq=False, repr=False)
class QueryUnreceivedAcksRequest(betterproto.Message):
    """
    QueryUnreceivedAcks is the request type for the
    Query/UnreceivedAcks RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""

    packet_ack_sequences: List[int] = betterproto.uint64_field(3)
    """list of acknowledgement sequences"""


@dataclass(eq=False, repr=False)
class QueryUnreceivedAcksResponse(betterproto.Message):
    """
    QueryUnreceivedAcksResponse is the response type for the
    Query/UnreceivedAcks RPC method
    """

    sequences: List[int] = betterproto.uint64_field(1)
    """list of unreceived acknowledgement sequences"""

    height: '__client_v1__.Height' = betterproto.message_field(2)
    """query block height"""


@dataclass(eq=False, repr=False)
class QueryNextSequenceReceiveRequest(betterproto.Message):
    """
    QueryNextSequenceReceiveRequest is the request type for the
    Query/QueryNextSequenceReceiveRequest RPC method
    """

    port_id: str = betterproto.string_field(1)
    """port unique identifier"""

    channel_id: str = betterproto.string_field(2)
    """channel unique identifier"""


@dataclass(eq=False, repr=False)
class QueryNextSequenceReceiveResponse(betterproto.Message):
    """
    QuerySequenceResponse is the request type for the
    Query/QueryNextSequenceReceiveResponse RPC method
    """

    next_sequence_receive: int = betterproto.uint64_field(1)
    """next sequence receive number"""

    proof: bytes = betterproto.bytes_field(2)
    """merkle proof of existence"""

    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    """height at which the proof was retrieved"""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the ibc channel submodule's genesis state."""

    channels: List['IdentifiedChannel'] = betterproto.message_field(1)
    acknowledgements: List['PacketState'] = betterproto.message_field(2)
    commitments: List['PacketState'] = betterproto.message_field(3)
    receipts: List['PacketState'] = betterproto.message_field(4)
    send_sequences: List['PacketSequence'] = betterproto.message_field(5)
    recv_sequences: List['PacketSequence'] = betterproto.message_field(6)
    ack_sequences: List['PacketSequence'] = betterproto.message_field(7)
    next_channel_sequence: int = betterproto.uint64_field(8)
    """the sequence for the next generated channel identifier"""


@dataclass(eq=False, repr=False)
class PacketSequence(betterproto.Message):
    """
    PacketSequence defines the genesis type necessary to retrieve and store
    next send and receive sequences.
    """

    port_id: str = betterproto.string_field(1)
    channel_id: str = betterproto.string_field(2)
    sequence: int = betterproto.uint64_field(3)


@dataclass(eq=False, repr=False)
class MsgChannelOpenInit(betterproto.Message):
    """
    MsgChannelOpenInit defines an sdk.Msg to initialize a channel handshake. It
    is called by a relayer on Chain A.
    """

    port_id: str = betterproto.string_field(1)
    channel: 'Channel' = betterproto.message_field(2)
    signer: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class MsgChannelOpenInitResponse(betterproto.Message):
    """
    MsgChannelOpenInitResponse defines the Msg/ChannelOpenInit response type.
    """

    channel_id: str = betterproto.string_field(1)


@dataclass(eq=False, repr=False)
class MsgChannelOpenTry(betterproto.Message):
    """
    MsgChannelOpenInit defines a msg sent by a Relayer to try to open a channel
    on Chain B. The version field within the Channel field has been deprecated.
    Its value will be ignored by core IBC.
    """

    port_id: str = betterproto.string_field(1)
    previous_channel_id: str = betterproto.string_field(2)
    """
    in the case of crossing hello's, when both chains call OpenInit, we need
    the channel identifier of the previous channel in state INIT
    """

    channel: 'Channel' = betterproto.message_field(3)
    """
    NOTE: the version field within the channel has been deprecated. Its value
    will be ignored by core IBC.
    """

    counterparty_version: str = betterproto.string_field(4)
    proof_init: bytes = betterproto.bytes_field(5)
    proof_height: '__client_v1__.Height' = betterproto.message_field(6)
    signer: str = betterproto.string_field(7)


@dataclass(eq=False, repr=False)
class MsgChannelOpenTryResponse(betterproto.Message):
    """
    MsgChannelOpenTryResponse defines the Msg/ChannelOpenTry response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgChannelOpenAck(betterproto.Message):
    """
    MsgChannelOpenAck defines a msg sent by a Relayer to Chain A to acknowledge
    the change of channel state to TRYOPEN on Chain B.
    """

    port_id: str = betterproto.string_field(1)
    channel_id: str = betterproto.string_field(2)
    counterparty_channel_id: str = betterproto.string_field(3)
    counterparty_version: str = betterproto.string_field(4)
    proof_try: bytes = betterproto.bytes_field(5)
    proof_height: '__client_v1__.Height' = betterproto.message_field(6)
    signer: str = betterproto.string_field(7)


@dataclass(eq=False, repr=False)
class MsgChannelOpenAckResponse(betterproto.Message):
    """
    MsgChannelOpenAckResponse defines the Msg/ChannelOpenAck response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgChannelOpenConfirm(betterproto.Message):
    """
    MsgChannelOpenConfirm defines a msg sent by a Relayer to Chain B to
    acknowledge the change of channel state to OPEN on Chain A.
    """

    port_id: str = betterproto.string_field(1)
    channel_id: str = betterproto.string_field(2)
    proof_ack: bytes = betterproto.bytes_field(3)
    proof_height: '__client_v1__.Height' = betterproto.message_field(4)
    signer: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class MsgChannelOpenConfirmResponse(betterproto.Message):
    """
    MsgChannelOpenConfirmResponse defines the Msg/ChannelOpenConfirm response
    type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgChannelCloseInit(betterproto.Message):
    """
    MsgChannelCloseInit defines a msg sent by a Relayer to Chain A
    to close a channel with Chain B.
    """

    port_id: str = betterproto.string_field(1)
    channel_id: str = betterproto.string_field(2)
    signer: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class MsgChannelCloseInitResponse(betterproto.Message):
    """
    MsgChannelCloseInitResponse defines the Msg/ChannelCloseInit response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgChannelCloseConfirm(betterproto.Message):
    """
    MsgChannelCloseConfirm defines a msg sent by a Relayer to Chain B
    to acknowledge the change of channel state to CLOSED on Chain A.
    """

    port_id: str = betterproto.string_field(1)
    channel_id: str = betterproto.string_field(2)
    proof_init: bytes = betterproto.bytes_field(3)
    proof_height: '__client_v1__.Height' = betterproto.message_field(4)
    signer: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class MsgChannelCloseConfirmResponse(betterproto.Message):
    """
    MsgChannelCloseConfirmResponse defines the Msg/ChannelCloseConfirm response
    type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgRecvPacket(betterproto.Message):
    """MsgRecvPacket receives incoming IBC packet"""

    packet: 'Packet' = betterproto.message_field(1)
    proof_commitment: bytes = betterproto.bytes_field(2)
    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    signer: str = betterproto.string_field(4)


@dataclass(eq=False, repr=False)
class MsgRecvPacketResponse(betterproto.Message):
    """MsgRecvPacketResponse defines the Msg/RecvPacket response type."""

    pass


@dataclass(eq=False, repr=False)
class MsgTimeout(betterproto.Message):
    """MsgTimeout receives timed-out packet"""

    packet: 'Packet' = betterproto.message_field(1)
    proof_unreceived: bytes = betterproto.bytes_field(2)
    proof_height: '__client_v1__.Height' = betterproto.message_field(3)
    next_sequence_recv: int = betterproto.uint64_field(4)
    signer: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class MsgTimeoutResponse(betterproto.Message):
    """MsgTimeoutResponse defines the Msg/Timeout response type."""

    pass


@dataclass(eq=False, repr=False)
class MsgTimeoutOnClose(betterproto.Message):
    """
    MsgTimeoutOnClose timed-out packet upon counterparty channel closure.
    """

    packet: 'Packet' = betterproto.message_field(1)
    proof_unreceived: bytes = betterproto.bytes_field(2)
    proof_close: bytes = betterproto.bytes_field(3)
    proof_height: '__client_v1__.Height' = betterproto.message_field(4)
    next_sequence_recv: int = betterproto.uint64_field(5)
    signer: str = betterproto.string_field(6)


@dataclass(eq=False, repr=False)
class MsgTimeoutOnCloseResponse(betterproto.Message):
    """
    MsgTimeoutOnCloseResponse defines the Msg/TimeoutOnClose response type.
    """

    pass


@dataclass(eq=False, repr=False)
class MsgAcknowledgement(betterproto.Message):
    """MsgAcknowledgement receives incoming IBC acknowledgement"""

    packet: 'Packet' = betterproto.message_field(1)
    acknowledgement: bytes = betterproto.bytes_field(2)
    proof_acked: bytes = betterproto.bytes_field(3)
    proof_height: '__client_v1__.Height' = betterproto.message_field(4)
    signer: str = betterproto.string_field(5)


@dataclass(eq=False, repr=False)
class MsgAcknowledgementResponse(betterproto.Message):
    """
    MsgAcknowledgementResponse defines the Msg/Acknowledgement response type.
    """

    pass


class QueryStub(betterproto.ServiceStub):
    async def channel(
        self,
        query_channel_request: 'QueryChannelRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryChannelResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/Channel',
            query_channel_request,
            QueryChannelResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def channels(
        self,
        query_channels_request: 'QueryChannelsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryChannelsResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/Channels',
            query_channels_request,
            QueryChannelsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def connection_channels(
        self,
        query_connection_channels_request: 'QueryConnectionChannelsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryConnectionChannelsResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/ConnectionChannels',
            query_connection_channels_request,
            QueryConnectionChannelsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def channel_client_state(
        self,
        query_channel_client_state_request: 'QueryChannelClientStateRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryChannelClientStateResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/ChannelClientState',
            query_channel_client_state_request,
            QueryChannelClientStateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def channel_consensus_state(
        self,
        query_channel_consensus_state_request: 'QueryChannelConsensusStateRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryChannelConsensusStateResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/ChannelConsensusState',
            query_channel_consensus_state_request,
            QueryChannelConsensusStateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def packet_commitment(
        self,
        query_packet_commitment_request: 'QueryPacketCommitmentRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryPacketCommitmentResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/PacketCommitment',
            query_packet_commitment_request,
            QueryPacketCommitmentResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def packet_commitments(
        self,
        query_packet_commitments_request: 'QueryPacketCommitmentsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryPacketCommitmentsResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/PacketCommitments',
            query_packet_commitments_request,
            QueryPacketCommitmentsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def packet_receipt(
        self,
        query_packet_receipt_request: 'QueryPacketReceiptRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryPacketReceiptResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/PacketReceipt',
            query_packet_receipt_request,
            QueryPacketReceiptResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def packet_acknowledgement(
        self,
        query_packet_acknowledgement_request: 'QueryPacketAcknowledgementRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryPacketAcknowledgementResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/PacketAcknowledgement',
            query_packet_acknowledgement_request,
            QueryPacketAcknowledgementResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def packet_acknowledgements(
        self,
        query_packet_acknowledgements_request: 'QueryPacketAcknowledgementsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryPacketAcknowledgementsResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/PacketAcknowledgements',
            query_packet_acknowledgements_request,
            QueryPacketAcknowledgementsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def unreceived_packets(
        self,
        query_unreceived_packets_request: 'QueryUnreceivedPacketsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryUnreceivedPacketsResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/UnreceivedPackets',
            query_unreceived_packets_request,
            QueryUnreceivedPacketsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def unreceived_acks(
        self,
        query_unreceived_acks_request: 'QueryUnreceivedAcksRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryUnreceivedAcksResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/UnreceivedAcks',
            query_unreceived_acks_request,
            QueryUnreceivedAcksResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def next_sequence_receive(
        self,
        query_next_sequence_receive_request: 'QueryNextSequenceReceiveRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryNextSequenceReceiveResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Query/NextSequenceReceive',
            query_next_sequence_receive_request,
            QueryNextSequenceReceiveResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgStub(betterproto.ServiceStub):
    async def channel_open_init(
        self,
        msg_channel_open_init: 'MsgChannelOpenInit',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgChannelOpenInitResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/ChannelOpenInit',
            msg_channel_open_init,
            MsgChannelOpenInitResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def channel_open_try(
        self,
        msg_channel_open_try: 'MsgChannelOpenTry',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgChannelOpenTryResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/ChannelOpenTry',
            msg_channel_open_try,
            MsgChannelOpenTryResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def channel_open_ack(
        self,
        msg_channel_open_ack: 'MsgChannelOpenAck',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgChannelOpenAckResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/ChannelOpenAck',
            msg_channel_open_ack,
            MsgChannelOpenAckResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def channel_open_confirm(
        self,
        msg_channel_open_confirm: 'MsgChannelOpenConfirm',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgChannelOpenConfirmResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/ChannelOpenConfirm',
            msg_channel_open_confirm,
            MsgChannelOpenConfirmResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def channel_close_init(
        self,
        msg_channel_close_init: 'MsgChannelCloseInit',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgChannelCloseInitResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/ChannelCloseInit',
            msg_channel_close_init,
            MsgChannelCloseInitResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def channel_close_confirm(
        self,
        msg_channel_close_confirm: 'MsgChannelCloseConfirm',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgChannelCloseConfirmResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/ChannelCloseConfirm',
            msg_channel_close_confirm,
            MsgChannelCloseConfirmResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def recv_packet(
        self,
        msg_recv_packet: 'MsgRecvPacket',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgRecvPacketResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/RecvPacket',
            msg_recv_packet,
            MsgRecvPacketResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def timeout(
        self,
        msg_timeout: 'MsgTimeout',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgTimeoutResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/Timeout',
            msg_timeout,
            MsgTimeoutResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def timeout_on_close(
        self,
        msg_timeout_on_close: 'MsgTimeoutOnClose',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgTimeoutOnCloseResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/TimeoutOnClose',
            msg_timeout_on_close,
            MsgTimeoutOnCloseResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def acknowledgement(
        self,
        msg_acknowledgement: 'MsgAcknowledgement',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgAcknowledgementResponse':
        return await self._unary_unary(
            '/ibc.core.channel.v1.Msg/Acknowledgement',
            msg_acknowledgement,
            MsgAcknowledgementResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def channel(
        self, query_channel_request: 'QueryChannelRequest'
    ) -> 'QueryChannelResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def channels(
        self, query_channels_request: 'QueryChannelsRequest'
    ) -> 'QueryChannelsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def connection_channels(
        self, query_connection_channels_request: 'QueryConnectionChannelsRequest'
    ) -> 'QueryConnectionChannelsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def channel_client_state(
        self, query_channel_client_state_request: 'QueryChannelClientStateRequest'
    ) -> 'QueryChannelClientStateResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def channel_consensus_state(
        self, query_channel_consensus_state_request: 'QueryChannelConsensusStateRequest'
    ) -> 'QueryChannelConsensusStateResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def packet_commitment(
        self, query_packet_commitment_request: 'QueryPacketCommitmentRequest'
    ) -> 'QueryPacketCommitmentResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def packet_commitments(
        self, query_packet_commitments_request: 'QueryPacketCommitmentsRequest'
    ) -> 'QueryPacketCommitmentsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def packet_receipt(
        self, query_packet_receipt_request: 'QueryPacketReceiptRequest'
    ) -> 'QueryPacketReceiptResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def packet_acknowledgement(
        self, query_packet_acknowledgement_request: 'QueryPacketAcknowledgementRequest'
    ) -> 'QueryPacketAcknowledgementResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def packet_acknowledgements(
        self,
        query_packet_acknowledgements_request: 'QueryPacketAcknowledgementsRequest',
    ) -> 'QueryPacketAcknowledgementsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def unreceived_packets(
        self, query_unreceived_packets_request: 'QueryUnreceivedPacketsRequest'
    ) -> 'QueryUnreceivedPacketsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def unreceived_acks(
        self, query_unreceived_acks_request: 'QueryUnreceivedAcksRequest'
    ) -> 'QueryUnreceivedAcksResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def next_sequence_receive(
        self, query_next_sequence_receive_request: 'QueryNextSequenceReceiveRequest'
    ) -> 'QueryNextSequenceReceiveResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_channel(
        self, stream: 'grpclib.server.Stream[QueryChannelRequest, QueryChannelResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.channel(request)
        await stream.send_message(response)

    async def __rpc_channels(
        self,
        stream: 'grpclib.server.Stream[QueryChannelsRequest, QueryChannelsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.channels(request)
        await stream.send_message(response)

    async def __rpc_connection_channels(
        self,
        stream: 'grpclib.server.Stream[QueryConnectionChannelsRequest, QueryConnectionChannelsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.connection_channels(request)
        await stream.send_message(response)

    async def __rpc_channel_client_state(
        self,
        stream: 'grpclib.server.Stream[QueryChannelClientStateRequest, QueryChannelClientStateResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.channel_client_state(request)
        await stream.send_message(response)

    async def __rpc_channel_consensus_state(
        self,
        stream: 'grpclib.server.Stream[QueryChannelConsensusStateRequest, QueryChannelConsensusStateResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.channel_consensus_state(request)
        await stream.send_message(response)

    async def __rpc_packet_commitment(
        self,
        stream: 'grpclib.server.Stream[QueryPacketCommitmentRequest, QueryPacketCommitmentResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.packet_commitment(request)
        await stream.send_message(response)

    async def __rpc_packet_commitments(
        self,
        stream: 'grpclib.server.Stream[QueryPacketCommitmentsRequest, QueryPacketCommitmentsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.packet_commitments(request)
        await stream.send_message(response)

    async def __rpc_packet_receipt(
        self,
        stream: 'grpclib.server.Stream[QueryPacketReceiptRequest, QueryPacketReceiptResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.packet_receipt(request)
        await stream.send_message(response)

    async def __rpc_packet_acknowledgement(
        self,
        stream: 'grpclib.server.Stream[QueryPacketAcknowledgementRequest, QueryPacketAcknowledgementResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.packet_acknowledgement(request)
        await stream.send_message(response)

    async def __rpc_packet_acknowledgements(
        self,
        stream: 'grpclib.server.Stream[QueryPacketAcknowledgementsRequest, QueryPacketAcknowledgementsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.packet_acknowledgements(request)
        await stream.send_message(response)

    async def __rpc_unreceived_packets(
        self,
        stream: 'grpclib.server.Stream[QueryUnreceivedPacketsRequest, QueryUnreceivedPacketsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.unreceived_packets(request)
        await stream.send_message(response)

    async def __rpc_unreceived_acks(
        self,
        stream: 'grpclib.server.Stream[QueryUnreceivedAcksRequest, QueryUnreceivedAcksResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.unreceived_acks(request)
        await stream.send_message(response)

    async def __rpc_next_sequence_receive(
        self,
        stream: 'grpclib.server.Stream[QueryNextSequenceReceiveRequest, QueryNextSequenceReceiveResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.next_sequence_receive(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/ibc.core.channel.v1.Query/Channel': grpclib.const.Handler(
                self.__rpc_channel,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryChannelRequest,
                QueryChannelResponse,
            ),
            '/ibc.core.channel.v1.Query/Channels': grpclib.const.Handler(
                self.__rpc_channels,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryChannelsRequest,
                QueryChannelsResponse,
            ),
            '/ibc.core.channel.v1.Query/ConnectionChannels': grpclib.const.Handler(
                self.__rpc_connection_channels,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryConnectionChannelsRequest,
                QueryConnectionChannelsResponse,
            ),
            '/ibc.core.channel.v1.Query/ChannelClientState': grpclib.const.Handler(
                self.__rpc_channel_client_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryChannelClientStateRequest,
                QueryChannelClientStateResponse,
            ),
            '/ibc.core.channel.v1.Query/ChannelConsensusState': grpclib.const.Handler(
                self.__rpc_channel_consensus_state,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryChannelConsensusStateRequest,
                QueryChannelConsensusStateResponse,
            ),
            '/ibc.core.channel.v1.Query/PacketCommitment': grpclib.const.Handler(
                self.__rpc_packet_commitment,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryPacketCommitmentRequest,
                QueryPacketCommitmentResponse,
            ),
            '/ibc.core.channel.v1.Query/PacketCommitments': grpclib.const.Handler(
                self.__rpc_packet_commitments,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryPacketCommitmentsRequest,
                QueryPacketCommitmentsResponse,
            ),
            '/ibc.core.channel.v1.Query/PacketReceipt': grpclib.const.Handler(
                self.__rpc_packet_receipt,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryPacketReceiptRequest,
                QueryPacketReceiptResponse,
            ),
            '/ibc.core.channel.v1.Query/PacketAcknowledgement': grpclib.const.Handler(
                self.__rpc_packet_acknowledgement,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryPacketAcknowledgementRequest,
                QueryPacketAcknowledgementResponse,
            ),
            '/ibc.core.channel.v1.Query/PacketAcknowledgements': grpclib.const.Handler(
                self.__rpc_packet_acknowledgements,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryPacketAcknowledgementsRequest,
                QueryPacketAcknowledgementsResponse,
            ),
            '/ibc.core.channel.v1.Query/UnreceivedPackets': grpclib.const.Handler(
                self.__rpc_unreceived_packets,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryUnreceivedPacketsRequest,
                QueryUnreceivedPacketsResponse,
            ),
            '/ibc.core.channel.v1.Query/UnreceivedAcks': grpclib.const.Handler(
                self.__rpc_unreceived_acks,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryUnreceivedAcksRequest,
                QueryUnreceivedAcksResponse,
            ),
            '/ibc.core.channel.v1.Query/NextSequenceReceive': grpclib.const.Handler(
                self.__rpc_next_sequence_receive,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryNextSequenceReceiveRequest,
                QueryNextSequenceReceiveResponse,
            ),
        }


class MsgBase(ServiceBase):
    async def channel_open_init(
        self, msg_channel_open_init: 'MsgChannelOpenInit'
    ) -> 'MsgChannelOpenInitResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def channel_open_try(
        self, msg_channel_open_try: 'MsgChannelOpenTry'
    ) -> 'MsgChannelOpenTryResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def channel_open_ack(
        self, msg_channel_open_ack: 'MsgChannelOpenAck'
    ) -> 'MsgChannelOpenAckResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def channel_open_confirm(
        self, msg_channel_open_confirm: 'MsgChannelOpenConfirm'
    ) -> 'MsgChannelOpenConfirmResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def channel_close_init(
        self, msg_channel_close_init: 'MsgChannelCloseInit'
    ) -> 'MsgChannelCloseInitResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def channel_close_confirm(
        self, msg_channel_close_confirm: 'MsgChannelCloseConfirm'
    ) -> 'MsgChannelCloseConfirmResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def recv_packet(
        self, msg_recv_packet: 'MsgRecvPacket'
    ) -> 'MsgRecvPacketResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def timeout(self, msg_timeout: 'MsgTimeout') -> 'MsgTimeoutResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def timeout_on_close(
        self, msg_timeout_on_close: 'MsgTimeoutOnClose'
    ) -> 'MsgTimeoutOnCloseResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def acknowledgement(
        self, msg_acknowledgement: 'MsgAcknowledgement'
    ) -> 'MsgAcknowledgementResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_channel_open_init(
        self,
        stream: 'grpclib.server.Stream[MsgChannelOpenInit, MsgChannelOpenInitResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.channel_open_init(request)
        await stream.send_message(response)

    async def __rpc_channel_open_try(
        self,
        stream: 'grpclib.server.Stream[MsgChannelOpenTry, MsgChannelOpenTryResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.channel_open_try(request)
        await stream.send_message(response)

    async def __rpc_channel_open_ack(
        self,
        stream: 'grpclib.server.Stream[MsgChannelOpenAck, MsgChannelOpenAckResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.channel_open_ack(request)
        await stream.send_message(response)

    async def __rpc_channel_open_confirm(
        self,
        stream: 'grpclib.server.Stream[MsgChannelOpenConfirm, MsgChannelOpenConfirmResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.channel_open_confirm(request)
        await stream.send_message(response)

    async def __rpc_channel_close_init(
        self,
        stream: 'grpclib.server.Stream[MsgChannelCloseInit, MsgChannelCloseInitResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.channel_close_init(request)
        await stream.send_message(response)

    async def __rpc_channel_close_confirm(
        self,
        stream: 'grpclib.server.Stream[MsgChannelCloseConfirm, MsgChannelCloseConfirmResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.channel_close_confirm(request)
        await stream.send_message(response)

    async def __rpc_recv_packet(
        self, stream: 'grpclib.server.Stream[MsgRecvPacket, MsgRecvPacketResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.recv_packet(request)
        await stream.send_message(response)

    async def __rpc_timeout(
        self, stream: 'grpclib.server.Stream[MsgTimeout, MsgTimeoutResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.timeout(request)
        await stream.send_message(response)

    async def __rpc_timeout_on_close(
        self,
        stream: 'grpclib.server.Stream[MsgTimeoutOnClose, MsgTimeoutOnCloseResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.timeout_on_close(request)
        await stream.send_message(response)

    async def __rpc_acknowledgement(
        self,
        stream: 'grpclib.server.Stream[MsgAcknowledgement, MsgAcknowledgementResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.acknowledgement(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/ibc.core.channel.v1.Msg/ChannelOpenInit': grpclib.const.Handler(
                self.__rpc_channel_open_init,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgChannelOpenInit,
                MsgChannelOpenInitResponse,
            ),
            '/ibc.core.channel.v1.Msg/ChannelOpenTry': grpclib.const.Handler(
                self.__rpc_channel_open_try,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgChannelOpenTry,
                MsgChannelOpenTryResponse,
            ),
            '/ibc.core.channel.v1.Msg/ChannelOpenAck': grpclib.const.Handler(
                self.__rpc_channel_open_ack,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgChannelOpenAck,
                MsgChannelOpenAckResponse,
            ),
            '/ibc.core.channel.v1.Msg/ChannelOpenConfirm': grpclib.const.Handler(
                self.__rpc_channel_open_confirm,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgChannelOpenConfirm,
                MsgChannelOpenConfirmResponse,
            ),
            '/ibc.core.channel.v1.Msg/ChannelCloseInit': grpclib.const.Handler(
                self.__rpc_channel_close_init,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgChannelCloseInit,
                MsgChannelCloseInitResponse,
            ),
            '/ibc.core.channel.v1.Msg/ChannelCloseConfirm': grpclib.const.Handler(
                self.__rpc_channel_close_confirm,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgChannelCloseConfirm,
                MsgChannelCloseConfirmResponse,
            ),
            '/ibc.core.channel.v1.Msg/RecvPacket': grpclib.const.Handler(
                self.__rpc_recv_packet,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgRecvPacket,
                MsgRecvPacketResponse,
            ),
            '/ibc.core.channel.v1.Msg/Timeout': grpclib.const.Handler(
                self.__rpc_timeout,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgTimeout,
                MsgTimeoutResponse,
            ),
            '/ibc.core.channel.v1.Msg/TimeoutOnClose': grpclib.const.Handler(
                self.__rpc_timeout_on_close,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgTimeoutOnClose,
                MsgTimeoutOnCloseResponse,
            ),
            '/ibc.core.channel.v1.Msg/Acknowledgement': grpclib.const.Handler(
                self.__rpc_acknowledgement,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgAcknowledgement,
                MsgAcknowledgementResponse,
            ),
        }



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: evmos/claims/v1/claims.proto, evmos/claims/v1/genesis.proto, evmos/claims/v1/query.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from datetime import (
    datetime,
    timedelta,
)
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ....cosmos.base import v1beta1 as ___cosmos_base_v1_beta1__
from ....cosmos.base.query import v1beta1 as ___cosmos_base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


class Action(betterproto.Enum):
    """
    Action defines the list of available actions to claim the airdrop tokens.
    """

    ACTION_UNSPECIFIED = 0
    """UNSPECIFIED defines an invalid action."""

    ACTION_VOTE = 1
    """VOTE defines a proposal vote."""

    ACTION_DELEGATE = 2
    """DELEGATE defines an staking delegation."""

    ACTION_EVM = 3
    """EVM defines an EVM transaction."""

    ACTION_IBC_TRANSFER = 4
    """IBC Transfer defines a fungible token transfer transaction via IBC."""


@dataclass(eq=False, repr=False)
class Claim(betterproto.Message):
    """
    Claim defines the action, completed flag and the remaining claimable amount
    for a given user. This is only used during client queries.
    """

    action: 'Action' = betterproto.enum_field(1)
    """action enum"""

    completed: bool = betterproto.bool_field(2)
    """true if the action has been completed"""

    claimable_amount: str = betterproto.string_field(3)
    """claimable token amount for the action. Zero if completed"""


@dataclass(eq=False, repr=False)
class ClaimsRecordAddress(betterproto.Message):
    """
    ClaimsRecordAddress is the claims metadata per address that is used at
    Genesis.
    """

    address: str = betterproto.string_field(1)
    """bech32 or hex address of claim user"""

    initial_claimable_amount: str = betterproto.string_field(2)
    """total initial claimable amount for the user"""

    actions_completed: List[bool] = betterproto.bool_field(3)
    """slice of the available actions completed"""


@dataclass(eq=False, repr=False)
class ClaimsRecord(betterproto.Message):
    """
    ClaimsRecord defines the initial claimable airdrop amount and the list of
    completed actions to claim the tokens.
    """

    initial_claimable_amount: str = betterproto.string_field(1)
    """total initial claimable amount for the user"""

    actions_completed: List[bool] = betterproto.bool_field(2)
    """slice of the available actions completed"""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState define the claims module's genesis state."""

    params: 'Params' = betterproto.message_field(1)
    """params defines all the parameters of the module."""

    claims_records: List['ClaimsRecordAddress'] = betterproto.message_field(2)
    """list of claim records with the corresponding airdrop recipient"""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params defines the claims module's parameters."""

    enable_claims: bool = betterproto.bool_field(1)
    """enable claiming process"""

    airdrop_start_time: datetime = betterproto.message_field(2)
    """timestamp of the airdrop start"""

    duration_until_decay: timedelta = betterproto.message_field(3)
    """duration until decay of claimable tokens begin"""

    duration_of_decay: timedelta = betterproto.message_field(4)
    """duration of the token claim decay period"""

    claims_denom: str = betterproto.string_field(5)
    """denom of claimable coin"""

    authorized_channels: List[str] = betterproto.string_field(6)
    """
    list of authorized channel identifiers that can perform address
    attestations via IBC.
    """

    evm_channels: List[str] = betterproto.string_field(7)
    """list of channel identifiers from EVM compatible chains"""


@dataclass(eq=False, repr=False)
class QueryTotalUnclaimedRequest(betterproto.Message):
    """
    QueryTotalUnclaimedRequest is the request type for the Query/TotalUnclaimed
    RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryTotalUnclaimedResponse(betterproto.Message):
    """
    QueryTotalUnclaimedResponse is the response type for the Query/TotalUnclaimed
    RPC method.
    """

    coins: List['___cosmos_base_v1_beta1__.Coin'] = betterproto.message_field(1)
    """coins defines the unclaimed coins"""


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC method.
    """

    params: 'Params' = betterproto.message_field(1)
    """params defines the parameters of the module."""


@dataclass(eq=False, repr=False)
class QueryClaimsRecordsRequest(betterproto.Message):
    """
    QueryClaimsRecordsRequest is the request type for the Query/ClaimsRecords RPC
    method.
    """

    pagination: '___cosmos_base_query_v1_beta1__.PageRequest' = (
        betterproto.message_field(1)
    )
    """pagination defines an optional pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryClaimsRecordsResponse(betterproto.Message):
    """
    QueryClaimsRecordsResponse is the response type for the Query/ClaimsRecords
    RPC method.
    """

    claims: List['ClaimsRecordAddress'] = betterproto.message_field(1)
    """claims defines all claims records"""

    pagination: '___cosmos_base_query_v1_beta1__.PageResponse' = (
        betterproto.message_field(2)
    )
    """pagination defines the pagination in the response."""


@dataclass(eq=False, repr=False)
class QueryClaimsRecordRequest(betterproto.Message):
    """
    QueryClaimsRecordRequest is the request type for the Query/ClaimsRecord RPC
    method.
    """

    address: str = betterproto.string_field(1)
    """address defines the user to query claims record for"""


@dataclass(eq=False, repr=False)
class QueryClaimsRecordResponse(betterproto.Message):
    """
    QueryClaimsRecordResponse is the response type for the Query/ClaimsRecord RPC
    method.
    """

    initial_claimable_amount: str = betterproto.string_field(1)
    """total initial claimable amount for the user"""

    claims: List['Claim'] = betterproto.message_field(2)
    """the claims of the user"""


class QueryStub(betterproto.ServiceStub):
    async def total_unclaimed(
        self,
        query_total_unclaimed_request: 'QueryTotalUnclaimedRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryTotalUnclaimedResponse':
        return await self._unary_unary(
            '/evmos.claims.v1.Query/TotalUnclaimed',
            query_total_unclaimed_request,
            QueryTotalUnclaimedResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def params(
        self,
        query_params_request: 'QueryParamsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryParamsResponse':
        return await self._unary_unary(
            '/evmos.claims.v1.Query/Params',
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def claims_records(
        self,
        query_claims_records_request: 'QueryClaimsRecordsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryClaimsRecordsResponse':
        return await self._unary_unary(
            '/evmos.claims.v1.Query/ClaimsRecords',
            query_claims_records_request,
            QueryClaimsRecordsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def claims_record(
        self,
        query_claims_record_request: 'QueryClaimsRecordRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryClaimsRecordResponse':
        return await self._unary_unary(
            '/evmos.claims.v1.Query/ClaimsRecord',
            query_claims_record_request,
            QueryClaimsRecordResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def total_unclaimed(
        self, query_total_unclaimed_request: 'QueryTotalUnclaimedRequest'
    ) -> 'QueryTotalUnclaimedResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def params(
        self, query_params_request: 'QueryParamsRequest'
    ) -> 'QueryParamsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def claims_records(
        self, query_claims_records_request: 'QueryClaimsRecordsRequest'
    ) -> 'QueryClaimsRecordsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def claims_record(
        self, query_claims_record_request: 'QueryClaimsRecordRequest'
    ) -> 'QueryClaimsRecordResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_total_unclaimed(
        self,
        stream: 'grpclib.server.Stream[QueryTotalUnclaimedRequest, QueryTotalUnclaimedResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.total_unclaimed(request)
        await stream.send_message(response)

    async def __rpc_params(
        self, stream: 'grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    async def __rpc_claims_records(
        self,
        stream: 'grpclib.server.Stream[QueryClaimsRecordsRequest, QueryClaimsRecordsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.claims_records(request)
        await stream.send_message(response)

    async def __rpc_claims_record(
        self,
        stream: 'grpclib.server.Stream[QueryClaimsRecordRequest, QueryClaimsRecordResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.claims_record(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.claims.v1.Query/TotalUnclaimed': grpclib.const.Handler(
                self.__rpc_total_unclaimed,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryTotalUnclaimedRequest,
                QueryTotalUnclaimedResponse,
            ),
            '/evmos.claims.v1.Query/Params': grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
            '/evmos.claims.v1.Query/ClaimsRecords': grpclib.const.Handler(
                self.__rpc_claims_records,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryClaimsRecordsRequest,
                QueryClaimsRecordsResponse,
            ),
            '/evmos.claims.v1.Query/ClaimsRecord': grpclib.const.Handler(
                self.__rpc_claims_record,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryClaimsRecordRequest,
                QueryClaimsRecordResponse,
            ),
        }



from __future__ import annotations

from typing import Any, Mapping, Sequence

from evmos.eip712 import (
    MSG_CANCEL_FEE_SPLIT_TYPES,
    MSG_REGISTER_FEE_SPLIT_TYPES,
    MSG_UPDATE_FEE_SPLIT_TYPES,
    create_msg_cancel_fee_split,
    create_msg_register_fee_split,
    create_msg_update_fee_split,
)
from evmos.proto import (
    MessageGenerated,
    MsgCancelFeeSplit,
    MsgRegisterFeeSplit,
    MsgUpdateFeeSplit,
)
from evmos.proto import create_msg_cancel_fee_split as proto_msg_cancel_fee_split
from evmos.proto import create_msg_register_fee_split as proto_msg_register_fee_split
from evmos.proto import create_msg_update_fee_split as proto_msg_update_fee_split
from evmos.transactions.common import to_generated

# msgCancelFeeSplit.ts


@to_generated(MSG_CANCEL_FEE_SPLIT_TYPES)
def create_tx_msg_cancel_fee_split(
    contract_address: str,
    deployer_address: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgCancelFeeSplit]]:
    """Create transaction with message for fee split cancellation."""
    # EIP712
    msg = create_msg_cancel_fee_split(
        contract_address,
        deployer_address,
    )

    # Cosmos
    msg_cosmos = proto_msg_cancel_fee_split(
        contract_address,
        deployer_address,
    )

    return msg, msg_cosmos


# msgRegisterFeeSplit.ts


@to_generated(MSG_REGISTER_FEE_SPLIT_TYPES)
def create_tx_msg_register_fee_split(
    contract_address: str,
    deployer_address: str,
    withdrawer_address: str,
    nonces: Sequence[int],
) -> tuple[Mapping[str, Any], MessageGenerated[MsgRegisterFeeSplit]]:
    """Create transaction with message for fee split registration."""
    # EIP712
    msg = create_msg_register_fee_split(
        contract_address,
        deployer_address,
        withdrawer_address,
        nonces,
    )

    # Cosmos
    msg_cosmos = proto_msg_register_fee_split(
        contract_address,
        deployer_address,
        withdrawer_address,
        nonces,
    )
    return msg, msg_cosmos


# msgUpdateFeeSplit.ts


@to_generated(MSG_UPDATE_FEE_SPLIT_TYPES)
def create_tx_msg_update_fee_split(
    contract_address: str,
    deployer_address: str,
    withdrawer_address: str,
    nonces: Sequence[int],
) -> tuple[Mapping[str, Any], MessageGenerated[MsgUpdateFeeSplit]]:
    """Create transaction with message for fee split update."""
    # EIP712
    msg = create_msg_update_fee_split(
        contract_address,
        deployer_address,
        withdrawer_address,
    )

    # Cosmos
    msg_cosmos = proto_msg_update_fee_split(
        contract_address,
        deployer_address,
        withdrawer_address,
    )

    return msg, msg_cosmos



from __future__ import annotations

from typing import Sequence

from evmos.proto.autogen.py.evmos.feesplit.v1 import (
    MsgCancelFeeSplit,
    MsgRegisterFeeSplit,
    MsgUpdateFeeSplit,
)
from evmos.proto.utils import MessageGenerated

# msgCancelFeeSplit.ts


def create_msg_cancel_fee_split(
    contract_address: str,
    deployer_address: str,
) -> MessageGenerated[MsgCancelFeeSplit]:
    """Create a message for fee split cancellation."""
    msg = MsgCancelFeeSplit(
        contract_address=contract_address,
        deployer_address=deployer_address,
    )
    return MessageGenerated(
        message=msg,
        path='evmos.feesplit.v1.MsgCancelFeeSplit',
    )


# msgRegisterFeeSplit.ts


def create_msg_register_fee_split(
    contract_address: str,
    deployer_address: str,
    withdrawer_address: str,
    nonces: Sequence[int],
) -> MessageGenerated[MsgRegisterFeeSplit]:
    """Create a message for fee split registration."""
    msg = MsgRegisterFeeSplit(
        contract_address=contract_address,
        deployer_address=deployer_address,
        withdrawer_address=withdrawer_address,
        nonces=list(nonces),
    )
    return MessageGenerated(
        message=msg,
        path='evmos.feesplit.v1.MsgRegisterFeeSplit',
    )


# msgUpdateFeeSplit.ts


def create_msg_update_fee_split(
    contract_address: str,
    deployer_address: str,
    withdrawer_address: str,
) -> MessageGenerated[MsgUpdateFeeSplit]:
    """Create a message for fee split update."""
    msg = MsgUpdateFeeSplit(
        contract_address=contract_address,
        deployer_address=deployer_address,
        withdrawer_address=withdrawer_address,
    )
    return MessageGenerated(
        message=msg,
        path='evmos.feesplit.v1.MsgUpdateFeeSplit',
    )



from __future__ import annotations

from evmos.proto import (
    MessageGenerated,
    MsgGrant,
    MsgRevoke,
    RevokeMessages,
    StakeAuthTypes,
    create_msg_grant,
    create_msg_revoke,
    create_stake_authorization,
)
from evmos.transactions.common import to_generated_base


@to_generated_base
def create_tx_msg_stake_authorization(
    account_address: str,
    bot_address: str,
    validator_address: str,
    denom: str,
    duration_in_seconds: int,
    max_tokens: str | None = None,
) -> MessageGenerated[MsgGrant]:
    """Create a transaction with message for stake authorization."""
    # EIP712
    # This is blocked until EvmosV7 is released with the eip712 any messages fixes!

    # Cosmos
    msg_stake_grant = create_stake_authorization(
        validator_address,
        denom,
        max_tokens,
        StakeAuthTypes.AUTHORIZATION_TYPE_DELEGATE,  # type: ignore
    )
    return create_msg_grant(
        account_address,
        bot_address,
        msg_stake_grant,
        duration_in_seconds,
    )


@to_generated_base
def create_tx_msg_stake_revoke_authorization(
    account_address: str,
    bot_address: str,
) -> MessageGenerated[MsgRevoke]:
    """Create a transaction with message for stake authorization revoke."""
    # EIP712
    # This is blocked until EvmosV7 is released with the eip712 any messages fixes!

    # Cosmos
    return create_msg_revoke(
        account_address,
        bot_address,
        RevokeMessages.REVOKE_MSG_DELEGATE,
    )



from __future__ import annotations

from enum import Enum
from typing import Final

from betterproto import Message, Timestamp

from evmos.proto.autogen.py.cosmos.authz.v1beta1 import Grant, MsgGrant, MsgRevoke
from evmos.proto.autogen.py.cosmos.base.v1beta1 import Coin
from evmos.proto.autogen.py.cosmos.staking.v1beta1 import (
    AuthorizationType,
    StakeAuthorization,
)
from evmos.proto.utils import MessageGenerated, create_any_message

# authz.ts


def create_msg_grant(
    granter: str,
    grantee: str,
    grant_message: MessageGenerated[Message],
    seconds: int,
) -> MessageGenerated[MsgGrant]:
    """Create grant message."""
    msg = MsgGrant(
        granter=granter,
        grantee=grantee,
        grant=Grant(
            authorization=create_any_message(grant_message),
            expiration=Timestamp(seconds=seconds, nanos=0),
        ),
    )
    return MessageGenerated(
        message=msg,
        path='cosmos.authz.v1beta1.MsgGrant',
    )


class RevokeMessages(str, Enum):
    """Revoke message paths."""

    REVOKE_MSG_DELEGATE = '/cosmos.staking.v1beta1.MsgDelegate'
    """Revoke delegate message path."""
    REVOKE_MSG_WITHDRAW_DELEGATOR_REWARDS = (
        '/cosmos.distribution.v1beta1.MsgWithdrawDelegatorReward'
    )
    """Revoke delegate and withdraw rewards message path."""


def create_msg_revoke(
    granter: str,
    grantee: str,
    type: str | RevokeMessages,  # noqa: A002
) -> MessageGenerated[MsgRevoke]:
    """Create revoke message."""
    msg = MsgRevoke(
        granter=granter,
        grantee=grantee,
        msg_type_url=type,
    )
    return MessageGenerated(
        message=msg,
        path='cosmos.authz.v1beta1.MsgRevoke',
    )


# stake.ts

StakeAuthTypes: Final = AuthorizationType
"""Alias for :class:`evmos.proto.autogen.py.cosmos.staking.v1beta1.AuthorizationType`"""


def create_stake_authorization(
    allow_address: str,
    denom: str,
    max_tokens: str | None,
    authorization_type: AuthorizationType,
) -> MessageGenerated[StakeAuthorization]:
    """Create staking authorization message."""
    msg = StakeAuthorization(
        allow_list=StakeAuthorization.Validators(address=[allow_address]),
        max_tokens=(
            Coin(denom=denom, amount=max_tokens)  # type: ignore[arg-type]
            if max_tokens
            else None
        ),
        authorization_type=authorization_type,
    )

    return MessageGenerated(
        message=msg,
        path='cosmos.staking.v1beta1.StakeAuthorization',
    )



from __future__ import annotations

from typing import Final

from evmos.constants import NOT_MODIFY
from evmos.eip712.base import MsgInterface

MSG_EDIT_VALIDATOR_TYPES: Final = {
    'TypeDescription': [
        {'name': 'moniker', 'type': 'string'},
        {'name': 'identity', 'type': 'string'},
        {'name': 'website', 'type': 'string'},
        {'name': 'security_contact', 'type': 'string'},
        {'name': 'details', 'type': 'string'},
    ],
    'MsgValue': [
        {'name': 'description', 'type': 'TypeDescription'},
        {'name': 'validator_address', 'type': 'string'},
        {'name': 'commission_rate', 'type': 'string'},
        {'name': 'min_self_delegation', 'type': 'string'},
    ],
}
"""Types for validator editing message."""


def create_msg_edit_validator(
    *,
    moniker: str | None,
    identity: str | None,
    website: str | None,
    security_contact: str | None,
    details: str | None,
    validator_address: str,
    commission_rate: int | str | None = None,
    min_self_delegation: int | str | None = None,
) -> MsgInterface:
    """Create validator editing message."""
    return {
        'type': 'cosmos-sdk/MsgEditValidator',
        'value': {
            'description': {
                'moniker': moniker or NOT_MODIFY,
                'identity': identity or NOT_MODIFY,
                'website': website or NOT_MODIFY,
                'security_contact': security_contact or NOT_MODIFY,
                'details': details or NOT_MODIFY,
            },
            'validator_address': validator_address,
            'commission_rate': str(commission_rate or '<nil>'),
            'min_self_delegation': str(min_self_delegation or '<nil>'),
        },
    }



from __future__ import annotations

from typing import Any, Mapping

from eth_typing import HexStr

from evmos.eip712 import (
    MSG_CONVERT_COIN_TYPES,
    MSG_CONVERT_ERC20_TYPES,
    create_msg_convert_coin,
    create_msg_convert_erc20,
)
from evmos.proto import MessageGenerated, MsgConvertCoin, MsgConvertErc20
from evmos.proto import create_msg_convert_coin as proto_msg_convert_coin
from evmos.proto import create_msg_convert_erc20 as proto_msg_convert_erc20
from evmos.transactions.common import to_generated

# msgConvertCoin.ts


@to_generated(MSG_CONVERT_COIN_TYPES)
def create_tx_msg_convert_coin(
    denom: str,
    amount: str,
    receiver_hex_formatted: HexStr,
    sender_evmos_formatted: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgConvertCoin]]:
    """Create transaction with message for coin conversion."""
    # EIP712
    msg = create_msg_convert_coin(
        denom,
        amount,
        receiver_hex_formatted,
        sender_evmos_formatted,
    )

    # Cosmos
    msg_cosmos = proto_msg_convert_coin(
        denom,
        amount,
        receiver_hex_formatted,
        sender_evmos_formatted,
    )

    return msg, msg_cosmos


# msgConvertERC20.ts


@to_generated(MSG_CONVERT_ERC20_TYPES)
def create_tx_msg_convert_erc20(
    contract_address: str,
    amount: str,
    receiver_evmos_formatted: str,
    sender_hex_formatted: str,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgConvertErc20]]:
    """Create transaction with message for ERC20 conversion."""
    # EIP712
    msg = create_msg_convert_erc20(
        contract_address,
        amount,
        receiver_evmos_formatted,
        sender_hex_formatted,
    )

    # Cosmos
    msg_cosmos = proto_msg_convert_erc20(
        contract_address,
        amount,
        receiver_evmos_formatted,
        sender_hex_formatted,
    )

    return msg, msg_cosmos



from __future__ import annotations

import base64
import warnings
from typing import Any, Final

import requests
from eth_account import Account
from eth_typing import HexStr
from eth_utils import keccak

from evmos.proto import create_tx_raw
from evmos.provider import BroadcastMode, BroadcastPostBody, generate_endpoint_broadcast
from evmos.transactions import (
    Chain,
    Fee,
    Sender,
    TxGenerated,
    create_tx_raw_eip712,
    signature_to_web3_extension,
)
from evmos.utils.eip_712_hash import hash_domain, hash_message

# Copied from
# https://github.com/hanchon-live/evmos-ts-wallet/blob/main/src/signer.ts
# with major modifications.


# Chain helpers

LOCALNET_CHAIN: Final = Chain(
    chain_id=9000,
    cosmos_chain_id='evmos_9000-1',
)

LOCALNET_FEE: Final = Fee(
    amount='2000000000000',
    denom='atevmos',
    gas='200000',
)

MAINNET_CHAIN: Final = Chain(
    chain_id=9001,
    cosmos_chain_id='evmos_9001-2',
)

MAINNET_FEE: Final = Fee(
    amount='6000000000000',
    denom='atevmos',
    gas='600000',
)

TESTNET_CHAIN: Final = Chain(
    chain_id=9000,
    cosmos_chain_id='evmos_9000-4',
)

TESTNET_FEE: Final = Fee(
    amount='6000000000000',
    denom='atevmos',
    gas='600000',
)


def broadcast(
    transaction_body: BroadcastPostBody,
    url: str = 'http://127.0.0.1:1317',
) -> dict[str, Any]:
    """Broadcast a transaction.

    Args:
        transaction_body: data to broadcast, json payload (not stringified).
        url: REST API URL to use.

    Returns:
        Info about broadcasted transaction or failure reasons.
    """
    post = requests.post(
        f'{url}{generate_endpoint_broadcast()}',
        json=transaction_body,
    )
    return post.json()


def sign_transaction(
    tx: TxGenerated,
    private_key: HexStr,
    broadcast_mode: BroadcastMode = BroadcastMode.BLOCK,
) -> BroadcastPostBody:
    """Sign transaction using payload method (keplr style)."""
    data_to_sign = base64.b64decode(tx.sign_direct.sign_bytes)

    with warnings.catch_warnings():
        # signHash is deprecated, but there is no alternative to sign raw bytes
        warnings.filterwarnings('ignore', category=DeprecationWarning)
        signature_raw = Account.signHash(data_to_sign, private_key=private_key)

    signed_tx = create_tx_raw(
        bytes(tx.sign_direct.body),
        bytes(tx.sign_direct.auth_info),
        [signature_raw.signature],
    )
    return {
        'tx_bytes': base64.b64encode(bytes(signed_tx.message)).decode(),
        'mode': 'BROADCAST_MODE_BLOCK',
    }


def sign_transaction_eip712(
    sender: Sender,
    tx: TxGenerated,
    private_key: HexStr,
    chain: Chain = TESTNET_CHAIN,
    broadcast_mode: BroadcastMode = BroadcastMode.BLOCK,
) -> BroadcastPostBody:
    """Sign transaction using eip712 method (metamask style)."""
    data_to_sign = keccak(
        b'\x19\x01' + hash_domain(tx.eip_to_sign) + hash_message(tx.eip_to_sign)
    )

    with warnings.catch_warnings():
        # signHash is deprecated, but there is no alternative to sign raw bytes
        warnings.filterwarnings('ignore', category=DeprecationWarning)
        signature_raw = Account.signHash(data_to_sign, private_key=private_key)

    signature = signature_raw.signature

    extension = signature_to_web3_extension(
        chain,
        sender,
        signature,
    )
    signed_tx = create_tx_raw_eip712(
        tx.legacy_amino.body,
        tx.legacy_amino.auth_info,
        extension,
    )

    return {
        'tx_bytes': base64.b64encode(bytes(signed_tx.message)).decode(),
        'mode': broadcast_mode,
    }



from __future__ import annotations

import re
from dataclasses import dataclass
from typing import Callable, Final, Generic, TypeVar

import bech32
from eth_typing import HexStr
from eth_utils import keccak

from evmos.utils.polyfill import removeprefix

ADDRESS_REGEX: Final = re.compile(r'^0x[0-9a-fA-F]{40}$')


def to_checksum_address(address: HexStr, chain_id: int | None = None) -> HexStr:
    """Convert address to checksum address."""
    strip_address = removeprefix(address, '0x').lower()
    prefix = (str(chain_id) + '0x') if chain_id is not None else ''
    keccak_hash = keccak((prefix + strip_address).encode()).hex()
    output = '0x' + ''.join(
        byte.upper() if int(hash_byte, 16) >= 8 else byte
        for byte, hash_byte in zip(strip_address, keccak_hash)
    )
    return HexStr(output)


def is_valid_address(address: HexStr) -> bool:
    """Check if address is in a proper format."""
    return bool(ADDRESS_REGEX.match(address))


def is_valid_checksum_address(address: HexStr, chain_id: int | None = None) -> bool:
    """Check if checksum address is valid."""
    return bool(
        is_valid_address(address)
        and (to_checksum_address(address, chain_id) == address)
    )


_T = TypeVar('_T')


@dataclass
class ChainConverter(Generic[_T]):
    """Chain decoder-encoder pair."""

    decoder: Callable[[_T], bytes]
    """Address decoder for given chain."""
    encoder: Callable[[bytes], _T]
    """Address encoder for given chain."""
    name: str
    """Chain name."""


def make_checksummed_hex_decoder(
    chain_id: int | None = None,
) -> Callable[[HexStr], bytes]:
    """Make decoder for hex-based address."""

    def decoder(data: HexStr) -> bytes:
        """Address decoder."""
        stripped = removeprefix(data, '0x')
        if (
            not is_valid_checksum_address(data, chain_id)
            and stripped != stripped.lower()
            and stripped != stripped.upper()
        ):
            raise ValueError('Invalid address checksum.')
        return bytes.fromhex(stripped)

    return decoder


def make_checksummed_hex_encoder(
    chain_id: int | None = None,
) -> Callable[[bytes], HexStr]:
    """Make encoder for hex-based address."""

    def encoder(data: bytes) -> HexStr:
        """Address encoder."""
        return to_checksum_address(HexStr(data.hex()), chain_id)

    return encoder


def hex_checksum_chain(
    name: str, chain_id: int | None = None
) -> ChainConverter[HexStr]:
    """Chain with hex address."""
    return ChainConverter(
        decoder=make_checksummed_hex_decoder(chain_id),
        encoder=make_checksummed_hex_encoder(chain_id),
        name=name,
    )


ETH: Final = hex_checksum_chain('ETH')
"""Eth chain address converter."""


def make_bech32_decoder(current_prefix: str) -> Callable[[str], bytes]:
    """Make decoder for bech32-based address."""

    def decoder(data: str) -> bytes:
        """Address decoder."""
        prefix, words = bech32.bech32_decode(data)
        if prefix != current_prefix:
            raise ValueError('Unrecognised address format')

        decoded = bech32.convertbits(words, 5, 8, False)
        if decoded is None or len(decoded) < 2 or len(decoded) > 40:
            raise ValueError('Unrecognised address format')
        return bytes(decoded)

    return decoder


def make_bech32_encoder(prefix: str) -> Callable[[bytes], str]:
    """Make encoder for bech32-based address."""

    def encoder(data: bytes) -> str:
        """Address encoder."""
        return bech32.bech32_encode(prefix, bech32.convertbits(data, 8, 5))

    return encoder


def bech32_chain(name: str, prefix: str) -> ChainConverter[str]:
    """Chain with bech32 address."""
    return ChainConverter(
        decoder=make_bech32_decoder(prefix),
        encoder=make_bech32_encoder(prefix),
        name=name,
    )


ETHERMINT: Final = bech32_chain('ETHERMINT', 'ethm')
"""Ethermint chain address converter."""


def eth_to_ethermint(eth_address: HexStr) -> str:
    """Eth -> Ethermint address conversion."""
    data = ETH.decoder(eth_address)
    return ETHERMINT.encoder(data)


def ethermint_to_eth(ethermint_address: str) -> HexStr:
    """Ethermint -> Eth address conversion."""
    data = ETHERMINT.decoder(ethermint_address)
    return ETH.encoder(data)


EVMOS: Final = bech32_chain('EVMOS', 'evmos')
"""Evmos chain address converter."""


def eth_to_evmos(eth_address: HexStr) -> str:
    """Eth -> Evmos address conversion."""
    data = ETH.decoder(eth_address)
    return EVMOS.encoder(data)


def evmos_to_eth(evmos_address: str) -> HexStr:
    """Evmos -> Eth address conversion."""
    data = EVMOS.decoder(evmos_address)
    return ETH.encoder(data)


OSMOSIS: Final = bech32_chain('OSMOSIS', 'osmo')
"""Osmosis chain address converter."""


def eth_to_osmosis(eth_address: HexStr) -> str:
    """Eth -> Osmosis address conversion."""
    data = ETH.decoder(eth_address)
    return OSMOSIS.encoder(data)


def osmosis_to_eth(evmos_address: str) -> HexStr:
    """Osmosis -> Eth address conversion."""
    data = OSMOSIS.decoder(evmos_address)
    return ETH.encoder(data)


COSMOS: Final = bech32_chain('COSMOS', 'cosmos')
"""Cosmos chain address converter."""


def eth_to_cosmos(eth_address: HexStr) -> str:
    """Eth -> Cosmos address conversion."""
    data = ETH.decoder(eth_address)
    return COSMOS.encoder(data)


def cosmos_to_eth(evmos_address: str) -> HexStr:
    """Cosmos -> Eth address conversion."""
    data = COSMOS.decoder(evmos_address)
    return ETH.encoder(data)


KYVE: Final = bech32_chain('KORELLIA', 'kyve')
"""Kyve chain address converter."""


def eth_to_kyve(eth_address: HexStr) -> str:
    """Eth -> Kyve address conversion."""
    data = ETH.decoder(eth_address)
    return KYVE.encoder(data)


def kyve_to_eth(kyve_address: str) -> HexStr:
    """Kyve -> Eth address conversion."""
    data = KYVE.decoder(kyve_address)
    return ETH.encoder(data)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: evmos/inflation/v1/genesis.proto, evmos/inflation/v1/inflation.proto, evmos/inflation/v1/query.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from typing import (
    TYPE_CHECKING,
    Dict,
    Optional,
)

import betterproto
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ....cosmos.base import v1beta1 as ___cosmos_base_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class InflationDistribution(betterproto.Message):
    """
    InflationDistribution defines the distribution in which inflation is
    allocated through minting on each epoch (staking, incentives, community). It
    excludes the team vesting distribution, as this is minted once at genesis.
    The initial InflationDistribution can be calculated from the Evmos Token
    Model like this:
    mintDistribution1 = distribution1 / (1 - teamVestingDistribution)
    0.5333333         = 40%           / (1 - 25%)
    """

    staking_rewards: str = betterproto.string_field(1)
    """
    staking_rewards defines the proportion of the minted minted_denom that is
    to be allocated as staking rewards
    """

    usage_incentives: str = betterproto.string_field(2)
    """
    usage_incentives defines the proportion of the minted minted_denom that is
    to be allocated to the incentives module address
    """

    community_pool: str = betterproto.string_field(3)
    """
    community_pool defines the proportion of the minted minted_denom that is to
    be allocated to the community pool
    """


@dataclass(eq=False, repr=False)
class ExponentialCalculation(betterproto.Message):
    """
    ExponentialCalculation holds factors to calculate exponential inflation on
    each period. Calculation reference:
    periodProvision = exponentialDecay       *  bondingIncentive
    f(x)            = (a * (1 - r) ^ x + c)  *  (1 + max_variance - bondedRatio *
    (max_variance / bonding_target))
    """

    a: str = betterproto.string_field(1)
    """initial value"""

    r: str = betterproto.string_field(2)
    """reduction factor"""

    c: str = betterproto.string_field(3)
    """long term inflation"""

    bonding_target: str = betterproto.string_field(4)
    """bonding target"""

    max_variance: str = betterproto.string_field(5)
    """max variance"""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the inflation module's genesis state."""

    params: 'Params' = betterproto.message_field(1)
    """params defines all the paramaters of the module."""

    period: int = betterproto.uint64_field(2)
    """amount of past periods, based on the epochs per period param"""

    epoch_identifier: str = betterproto.string_field(3)
    """inflation epoch identifier"""

    epochs_per_period: int = betterproto.int64_field(4)
    """number of epochs after which inflation is recalculated"""

    skipped_epochs: int = betterproto.uint64_field(5)
    """number of epochs that have passed while inflation is disabled"""


@dataclass(eq=False, repr=False)
class Params(betterproto.Message):
    """Params holds parameters for the inflation module."""

    mint_denom: str = betterproto.string_field(1)
    """type of coin to mint"""

    exponential_calculation: 'ExponentialCalculation' = betterproto.message_field(2)
    """variables to calculate exponential inflation"""

    inflation_distribution: 'InflationDistribution' = betterproto.message_field(3)
    """inflation distribution of the minted denom"""

    enable_inflation: bool = betterproto.bool_field(4)
    """parameter to enable inflation and halt increasing the skipped_epochs"""


@dataclass(eq=False, repr=False)
class QueryPeriodRequest(betterproto.Message):
    """
    QueryPeriodRequest is the request type for the Query/Period RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryPeriodResponse(betterproto.Message):
    """
    QueryPeriodResponse is the response type for the Query/Period RPC method.
    """

    period: int = betterproto.uint64_field(1)
    """period is the current minting per epoch provision value."""


@dataclass(eq=False, repr=False)
class QueryEpochMintProvisionRequest(betterproto.Message):
    """
    QueryEpochMintProvisionRequest is the request type for the
    Query/EpochMintProvision RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryEpochMintProvisionResponse(betterproto.Message):
    """
    QueryEpochMintProvisionResponse is the response type for the
    Query/EpochMintProvision RPC method.
    """

    epoch_mint_provision: '___cosmos_base_v1_beta1__.DecCoin' = (
        betterproto.message_field(1)
    )
    """
    epoch_mint_provision is the current minting per epoch provision value.
    """


@dataclass(eq=False, repr=False)
class QuerySkippedEpochsRequest(betterproto.Message):
    """
    QuerySkippedEpochsRequest is the request type for the Query/SkippedEpochs RPC
    method.
    """

    pass


@dataclass(eq=False, repr=False)
class QuerySkippedEpochsResponse(betterproto.Message):
    """
    QuerySkippedEpochsResponse is the response type for the Query/SkippedEpochs
    RPC method.
    """

    skipped_epochs: int = betterproto.uint64_field(1)
    """number of epochs that the inflation module has been disabled."""


@dataclass(eq=False, repr=False)
class QueryCirculatingSupplyRequest(betterproto.Message):
    """
    QueryCirculatingSupplyRequest is the request type for the
    Query/CirculatingSupply RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryCirculatingSupplyResponse(betterproto.Message):
    """
    QueryCirculatingSupplyResponse is the response type for the
    Query/CirculatingSupply RPC method.
    """

    circulating_supply: '___cosmos_base_v1_beta1__.DecCoin' = betterproto.message_field(
        1
    )
    """total amount of coins in circulation"""


@dataclass(eq=False, repr=False)
class QueryInflationRateRequest(betterproto.Message):
    """
    QueryInflationRateRequest is the request type for the Query/InflationRate RPC
    method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryInflationRateResponse(betterproto.Message):
    """
    QueryInflationRateResponse is the response type for the Query/InflationRate
    RPC method.
    """

    inflation_rate: str = betterproto.string_field(1)
    """rate by which the total supply increases within one period"""


@dataclass(eq=False, repr=False)
class QueryParamsRequest(betterproto.Message):
    """
    QueryParamsRequest is the request type for the Query/Params RPC method.
    """

    pass


@dataclass(eq=False, repr=False)
class QueryParamsResponse(betterproto.Message):
    """
    QueryParamsResponse is the response type for the Query/Params RPC method.
    """

    params: 'Params' = betterproto.message_field(1)
    """params defines the parameters of the module."""


class QueryStub(betterproto.ServiceStub):
    async def period(
        self,
        query_period_request: 'QueryPeriodRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryPeriodResponse':
        return await self._unary_unary(
            '/evmos.inflation.v1.Query/Period',
            query_period_request,
            QueryPeriodResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def epoch_mint_provision(
        self,
        query_epoch_mint_provision_request: 'QueryEpochMintProvisionRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryEpochMintProvisionResponse':
        return await self._unary_unary(
            '/evmos.inflation.v1.Query/EpochMintProvision',
            query_epoch_mint_provision_request,
            QueryEpochMintProvisionResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def skipped_epochs(
        self,
        query_skipped_epochs_request: 'QuerySkippedEpochsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QuerySkippedEpochsResponse':
        return await self._unary_unary(
            '/evmos.inflation.v1.Query/SkippedEpochs',
            query_skipped_epochs_request,
            QuerySkippedEpochsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def circulating_supply(
        self,
        query_circulating_supply_request: 'QueryCirculatingSupplyRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryCirculatingSupplyResponse':
        return await self._unary_unary(
            '/evmos.inflation.v1.Query/CirculatingSupply',
            query_circulating_supply_request,
            QueryCirculatingSupplyResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def inflation_rate(
        self,
        query_inflation_rate_request: 'QueryInflationRateRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryInflationRateResponse':
        return await self._unary_unary(
            '/evmos.inflation.v1.Query/InflationRate',
            query_inflation_rate_request,
            QueryInflationRateResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def params(
        self,
        query_params_request: 'QueryParamsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryParamsResponse':
        return await self._unary_unary(
            '/evmos.inflation.v1.Query/Params',
            query_params_request,
            QueryParamsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def period(
        self, query_period_request: 'QueryPeriodRequest'
    ) -> 'QueryPeriodResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def epoch_mint_provision(
        self, query_epoch_mint_provision_request: 'QueryEpochMintProvisionRequest'
    ) -> 'QueryEpochMintProvisionResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def skipped_epochs(
        self, query_skipped_epochs_request: 'QuerySkippedEpochsRequest'
    ) -> 'QuerySkippedEpochsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def circulating_supply(
        self, query_circulating_supply_request: 'QueryCirculatingSupplyRequest'
    ) -> 'QueryCirculatingSupplyResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def inflation_rate(
        self, query_inflation_rate_request: 'QueryInflationRateRequest'
    ) -> 'QueryInflationRateResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def params(
        self, query_params_request: 'QueryParamsRequest'
    ) -> 'QueryParamsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_period(
        self, stream: 'grpclib.server.Stream[QueryPeriodRequest, QueryPeriodResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.period(request)
        await stream.send_message(response)

    async def __rpc_epoch_mint_provision(
        self,
        stream: 'grpclib.server.Stream[QueryEpochMintProvisionRequest, QueryEpochMintProvisionResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.epoch_mint_provision(request)
        await stream.send_message(response)

    async def __rpc_skipped_epochs(
        self,
        stream: 'grpclib.server.Stream[QuerySkippedEpochsRequest, QuerySkippedEpochsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.skipped_epochs(request)
        await stream.send_message(response)

    async def __rpc_circulating_supply(
        self,
        stream: 'grpclib.server.Stream[QueryCirculatingSupplyRequest, QueryCirculatingSupplyResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.circulating_supply(request)
        await stream.send_message(response)

    async def __rpc_inflation_rate(
        self,
        stream: 'grpclib.server.Stream[QueryInflationRateRequest, QueryInflationRateResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.inflation_rate(request)
        await stream.send_message(response)

    async def __rpc_params(
        self, stream: 'grpclib.server.Stream[QueryParamsRequest, QueryParamsResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.params(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/evmos.inflation.v1.Query/Period': grpclib.const.Handler(
                self.__rpc_period,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryPeriodRequest,
                QueryPeriodResponse,
            ),
            '/evmos.inflation.v1.Query/EpochMintProvision': grpclib.const.Handler(
                self.__rpc_epoch_mint_provision,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryEpochMintProvisionRequest,
                QueryEpochMintProvisionResponse,
            ),
            '/evmos.inflation.v1.Query/SkippedEpochs': grpclib.const.Handler(
                self.__rpc_skipped_epochs,
                grpclib.const.Cardinality.UNARY_UNARY,
                QuerySkippedEpochsRequest,
                QuerySkippedEpochsResponse,
            ),
            '/evmos.inflation.v1.Query/CirculatingSupply': grpclib.const.Handler(
                self.__rpc_circulating_supply,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryCirculatingSupplyRequest,
                QueryCirculatingSupplyResponse,
            ),
            '/evmos.inflation.v1.Query/InflationRate': grpclib.const.Handler(
                self.__rpc_inflation_rate,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryInflationRateRequest,
                QueryInflationRateResponse,
            ),
            '/evmos.inflation.v1.Query/Params': grpclib.const.Handler(
                self.__rpc_params,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryParamsRequest,
                QueryParamsResponse,
            ),
        }



from __future__ import annotations

from typing import Any, Mapping

from evmos.eip712 import MSG_EDIT_VALIDATOR_TYPES, create_msg_edit_validator
from evmos.proto import MessageGenerated, MsgEditValidator
from evmos.proto import create_msg_edit_validator as proto_msg_edit_validator
from evmos.transactions.common import to_generated


@to_generated(MSG_EDIT_VALIDATOR_TYPES)
def create_tx_msg_edit_validator(
    *,
    validator_address: str,  # FIXME: was str|None, but not allowed deeper
    moniker: str | None = None,
    identity: str | None = None,
    website: str | None = None,
    security_contact: str | None = None,
    details: str | None = None,
    commission_rate: str | None = None,
    min_self_delegation: str | None = None,
) -> tuple[Mapping[str, Any], MessageGenerated[MsgEditValidator]]:
    """Create a transaction with validator editing message."""
    # EIP712
    msg = create_msg_edit_validator(
        moniker=moniker,
        identity=identity,
        website=website,
        security_contact=security_contact,
        details=details,
        validator_address=validator_address,
        commission_rate=commission_rate,
        min_self_delegation=min_self_delegation,
    )

    # Cosmos
    msg_cosmos = proto_msg_edit_validator(
        moniker=moniker,
        identity=identity,
        website=website,
        security_contact=security_contact,
        details=details,
        validator_address=validator_address,
        commission_rate=commission_rate,
        min_self_delegation=min_self_delegation,
    )

    return msg, msg_cosmos



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/authz/v1beta1/authz.proto, cosmos/authz/v1beta1/event.proto, cosmos/authz/v1beta1/genesis.proto, cosmos/authz/v1beta1/query.proto, cosmos/authz/v1beta1/tx.proto
# plugin: python-betterproto
# This file has been @generated
from dataclasses import dataclass
from datetime import datetime
from typing import (
    TYPE_CHECKING,
    Dict,
    List,
    Optional,
)

import betterproto
import betterproto.lib.google.protobuf as betterproto_lib_google_protobuf
import grpclib
from betterproto.grpc.grpclib_server import ServiceBase

from ...base.query import v1beta1 as __base_query_v1_beta1__


if TYPE_CHECKING:
    import grpclib.server
    from betterproto.grpc.grpclib_client import MetadataLike
    from grpclib.metadata import Deadline


@dataclass(eq=False, repr=False)
class EventGrant(betterproto.Message):
    """EventGrant is emitted on Msg/Grant"""

    msg_type_url: str = betterproto.string_field(2)
    """Msg type URL for which an autorization is granted"""

    granter: str = betterproto.string_field(3)
    """Granter account address"""

    grantee: str = betterproto.string_field(4)
    """Grantee account address"""


@dataclass(eq=False, repr=False)
class EventRevoke(betterproto.Message):
    """EventRevoke is emitted on Msg/Revoke"""

    msg_type_url: str = betterproto.string_field(2)
    """Msg type URL for which an autorization is revoked"""

    granter: str = betterproto.string_field(3)
    """Granter account address"""

    grantee: str = betterproto.string_field(4)
    """Grantee account address"""


@dataclass(eq=False, repr=False)
class GenericAuthorization(betterproto.Message):
    """
    GenericAuthorization gives the grantee unrestricted permissions to execute
    the provided method on behalf of the granter's account.
    """

    msg: str = betterproto.string_field(1)
    """
    Msg, identified by it's type URL, to grant unrestricted permissions to execute
    """


@dataclass(eq=False, repr=False)
class Grant(betterproto.Message):
    """
    Grant gives permissions to execute
    the provide method with expiration time.
    """

    authorization: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(1)
    expiration: datetime = betterproto.message_field(2)


@dataclass(eq=False, repr=False)
class GrantAuthorization(betterproto.Message):
    """
    GrantAuthorization extends a grant with both the addresses of the grantee and
    granter.
    It is used in genesis.proto and query.proto
    Since: cosmos-sdk 0.45.2
    """

    granter: str = betterproto.string_field(1)
    grantee: str = betterproto.string_field(2)
    authorization: 'betterproto_lib_google_protobuf.Any' = betterproto.message_field(3)
    expiration: datetime = betterproto.message_field(4)


@dataclass(eq=False, repr=False)
class QueryGrantsRequest(betterproto.Message):
    """
    QueryGrantsRequest is the request type for the Query/Grants RPC method.
    """

    granter: str = betterproto.string_field(1)
    grantee: str = betterproto.string_field(2)
    msg_type_url: str = betterproto.string_field(3)
    """
    Optional, msg_type_url, when set, will query only grants matching given msg type.
    """

    pagination: '__base_query_v1_beta1__.PageRequest' = betterproto.message_field(4)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryGrantsResponse(betterproto.Message):
    """
    QueryGrantsResponse is the response type for the Query/Authorizations RPC method.
    """

    grants: List['Grant'] = betterproto.message_field(1)
    """authorizations is a list of grants granted for grantee by granter."""

    pagination: '__base_query_v1_beta1__.PageResponse' = betterproto.message_field(2)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class QueryGranterGrantsRequest(betterproto.Message):
    """
    QueryGranterGrantsRequest is the request type for the Query/GranterGrants RPC
    method.
    """

    granter: str = betterproto.string_field(1)
    pagination: '__base_query_v1_beta1__.PageRequest' = betterproto.message_field(2)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryGranterGrantsResponse(betterproto.Message):
    """
    QueryGranterGrantsResponse is the response type for the Query/GranterGrants RPC
    method.
    """

    grants: List['GrantAuthorization'] = betterproto.message_field(1)
    """grants is a list of grants granted by the granter."""

    pagination: '__base_query_v1_beta1__.PageResponse' = betterproto.message_field(2)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class QueryGranteeGrantsRequest(betterproto.Message):
    """
    QueryGranteeGrantsRequest is the request type for the Query/IssuedGrants RPC
    method.
    """

    grantee: str = betterproto.string_field(1)
    pagination: '__base_query_v1_beta1__.PageRequest' = betterproto.message_field(2)
    """pagination defines an pagination for the request."""


@dataclass(eq=False, repr=False)
class QueryGranteeGrantsResponse(betterproto.Message):
    """
    QueryGranteeGrantsResponse is the response type for the Query/GranteeGrants RPC
    method.
    """

    grants: List['GrantAuthorization'] = betterproto.message_field(1)
    """grants is a list of grants granted to the grantee."""

    pagination: '__base_query_v1_beta1__.PageResponse' = betterproto.message_field(2)
    """pagination defines an pagination for the response."""


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the authz module's genesis state."""

    authorization: List['GrantAuthorization'] = betterproto.message_field(1)


@dataclass(eq=False, repr=False)
class MsgGrant(betterproto.Message):
    """
    MsgGrant is a request type for Grant method. It declares authorization to the
    grantee
    on behalf of the granter with the provided expiration time.
    """

    granter: str = betterproto.string_field(1)
    grantee: str = betterproto.string_field(2)
    grant: 'Grant' = betterproto.message_field(3)


@dataclass(eq=False, repr=False)
class MsgExecResponse(betterproto.Message):
    """MsgExecResponse defines the Msg/MsgExecResponse response type."""

    results: List[bytes] = betterproto.bytes_field(1)


@dataclass(eq=False, repr=False)
class MsgExec(betterproto.Message):
    """
    MsgExec attempts to execute the provided messages using
    authorizations granted to the grantee. Each message should have only
    one signer corresponding to the granter of the authorization.
    """

    grantee: str = betterproto.string_field(1)
    msgs: List['betterproto_lib_google_protobuf.Any'] = betterproto.message_field(2)
    """
    Authorization Msg requests to execute. Each msg must implement Authorization
    interface
    The x/authz will try to find a grant matching (msg.signers[0], grantee,
    MsgTypeURL(msg))
    triple and validate it.
    """


@dataclass(eq=False, repr=False)
class MsgGrantResponse(betterproto.Message):
    """MsgGrantResponse defines the Msg/MsgGrant response type."""

    pass


@dataclass(eq=False, repr=False)
class MsgRevoke(betterproto.Message):
    """
    MsgRevoke revokes any authorization with the provided sdk.Msg type on the
    granter's account with that has been granted to the grantee.
    """

    granter: str = betterproto.string_field(1)
    grantee: str = betterproto.string_field(2)
    msg_type_url: str = betterproto.string_field(3)


@dataclass(eq=False, repr=False)
class MsgRevokeResponse(betterproto.Message):
    """MsgRevokeResponse defines the Msg/MsgRevokeResponse response type."""

    pass


class QueryStub(betterproto.ServiceStub):
    async def grants(
        self,
        query_grants_request: 'QueryGrantsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryGrantsResponse':
        return await self._unary_unary(
            '/cosmos.authz.v1beta1.Query/Grants',
            query_grants_request,
            QueryGrantsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def granter_grants(
        self,
        query_granter_grants_request: 'QueryGranterGrantsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryGranterGrantsResponse':
        return await self._unary_unary(
            '/cosmos.authz.v1beta1.Query/GranterGrants',
            query_granter_grants_request,
            QueryGranterGrantsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def grantee_grants(
        self,
        query_grantee_grants_request: 'QueryGranteeGrantsRequest',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'QueryGranteeGrantsResponse':
        return await self._unary_unary(
            '/cosmos.authz.v1beta1.Query/GranteeGrants',
            query_grantee_grants_request,
            QueryGranteeGrantsResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class MsgStub(betterproto.ServiceStub):
    async def grant(
        self,
        msg_grant: 'MsgGrant',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgGrantResponse':
        return await self._unary_unary(
            '/cosmos.authz.v1beta1.Msg/Grant',
            msg_grant,
            MsgGrantResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def exec(
        self,
        msg_exec: 'MsgExec',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgExecResponse':
        return await self._unary_unary(
            '/cosmos.authz.v1beta1.Msg/Exec',
            msg_exec,
            MsgExecResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )

    async def revoke(
        self,
        msg_revoke: 'MsgRevoke',
        *,
        timeout: Optional[float] = None,
        deadline: Optional['Deadline'] = None,
        metadata: Optional['MetadataLike'] = None
    ) -> 'MsgRevokeResponse':
        return await self._unary_unary(
            '/cosmos.authz.v1beta1.Msg/Revoke',
            msg_revoke,
            MsgRevokeResponse,
            timeout=timeout,
            deadline=deadline,
            metadata=metadata,
        )


class QueryBase(ServiceBase):
    async def grants(
        self, query_grants_request: 'QueryGrantsRequest'
    ) -> 'QueryGrantsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def granter_grants(
        self, query_granter_grants_request: 'QueryGranterGrantsRequest'
    ) -> 'QueryGranterGrantsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def grantee_grants(
        self, query_grantee_grants_request: 'QueryGranteeGrantsRequest'
    ) -> 'QueryGranteeGrantsResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_grants(
        self, stream: 'grpclib.server.Stream[QueryGrantsRequest, QueryGrantsResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.grants(request)
        await stream.send_message(response)

    async def __rpc_granter_grants(
        self,
        stream: 'grpclib.server.Stream[QueryGranterGrantsRequest, QueryGranterGrantsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.granter_grants(request)
        await stream.send_message(response)

    async def __rpc_grantee_grants(
        self,
        stream: 'grpclib.server.Stream[QueryGranteeGrantsRequest, QueryGranteeGrantsResponse]',
    ) -> None:
        request = await stream.recv_message()
        response = await self.grantee_grants(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/cosmos.authz.v1beta1.Query/Grants': grpclib.const.Handler(
                self.__rpc_grants,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryGrantsRequest,
                QueryGrantsResponse,
            ),
            '/cosmos.authz.v1beta1.Query/GranterGrants': grpclib.const.Handler(
                self.__rpc_granter_grants,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryGranterGrantsRequest,
                QueryGranterGrantsResponse,
            ),
            '/cosmos.authz.v1beta1.Query/GranteeGrants': grpclib.const.Handler(
                self.__rpc_grantee_grants,
                grpclib.const.Cardinality.UNARY_UNARY,
                QueryGranteeGrantsRequest,
                QueryGranteeGrantsResponse,
            ),
        }


class MsgBase(ServiceBase):
    async def grant(self, msg_grant: 'MsgGrant') -> 'MsgGrantResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def exec(self, msg_exec: 'MsgExec') -> 'MsgExecResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def revoke(self, msg_revoke: 'MsgRevoke') -> 'MsgRevokeResponse':
        raise grpclib.GRPCError(grpclib.const.Status.UNIMPLEMENTED)

    async def __rpc_grant(
        self, stream: 'grpclib.server.Stream[MsgGrant, MsgGrantResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.grant(request)
        await stream.send_message(response)

    async def __rpc_exec(
        self, stream: 'grpclib.server.Stream[MsgExec, MsgExecResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.exec(request)
        await stream.send_message(response)

    async def __rpc_revoke(
        self, stream: 'grpclib.server.Stream[MsgRevoke, MsgRevokeResponse]'
    ) -> None:
        request = await stream.recv_message()
        response = await self.revoke(request)
        await stream.send_message(response)

    def __mapping__(self) -> Dict[str, grpclib.const.Handler]:
        return {
            '/cosmos.authz.v1beta1.Msg/Grant': grpclib.const.Handler(
                self.__rpc_grant,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgGrant,
                MsgGrantResponse,
            ),
            '/cosmos.authz.v1beta1.Msg/Exec': grpclib.const.Handler(
                self.__rpc_exec,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgExec,
                MsgExecResponse,
            ),
            '/cosmos.authz.v1beta1.Msg/Revoke': grpclib.const.Handler(
                self.__rpc_revoke,
                grpclib.const.Cardinality.UNARY_UNARY,
                MsgRevoke,
                MsgRevokeResponse,
            ),
        }
